Module HiResWindowImageTools
    ' Produce a high resolution printout of the form.
    Public Sub HiResPrint(ByVal frm As Form, ByVal e As System.Drawing.Printing.PrintPageEventArgs, Optional ByVal enlarge_to_fit As Boolean = False, Optional ByVal draw_box As Boolean = True)
        '' Debugging: Draw the margins in printer units.
        'e.Graphics.DrawRectangle(New Pen(Color.Blue, 5), _
        '    e.MarginBounds.Left, _
        '    e.MarginBounds.Top, _
        '    e.MarginBounds.Width, _
        '    e.MarginBounds.Height)

        Dim scale As Single
        Dim x As Single
        Dim y As Single

        ' Set an appropriate scale transformation.
        If Not enlarge_to_fit Then
            ' Print at normal scale.
            ' Get the form client area's size 
            ' in 100ths of inches (the printer's units).
            Dim bm As New Bitmap(1, 1)
            scale = 100 / bm.HorizontalResolution
            bm = Nothing
            Dim wid As Single = frm.ClientSize.Width * scale
            Dim hgt As Single = frm.ClientSize.Height * scale

            ' Calculate the upper left corner
            ' in printer units.
            x = e.MarginBounds.Left + (e.MarginBounds.Width - wid) / 2
            y = e.MarginBounds.Top + (e.MarginBounds.Height - hgt) / 2
        Else
            ' Enlarge the image as much as possible
            ' but preserve the aspect ratio.
            Dim wid As Integer = frm.ClientSize.Width
            Dim hgt As Integer = frm.ClientSize.Height
            If wid / hgt > e.MarginBounds.Width / e.MarginBounds.Height Then
                ' The image is relatively short and wide.
                ' Make it as wide as possible.
                scale = CSng(e.MarginBounds.Width / wid)
            Else
                ' The image is relatively tall and thin.
                ' Make it as tall as possible.
                scale = CSng(e.MarginBounds.Height / hgt)
            End If
            wid = CInt(wid * scale)
            hgt = CInt(hgt * scale)

            ' Calculate the upper left corner 
            ' in printer units.
            x = e.MarginBounds.Left + (e.MarginBounds.Width - wid) \ 2
            y = e.MarginBounds.Top + (e.MarginBounds.Height - hgt) \ 2
        End If

        ' Set the transformation.
        e.Graphics.ScaleTransform(scale, scale, Drawing2D.MatrixOrder.Append)
        e.Graphics.TranslateTransform(x, y, Drawing2D.MatrixOrder.Append)

        '' Debugging: Draw the margins again in form units.
        'e.Graphics.DrawRectangle(Pens.Red, _
        '    (e.MarginBounds.Left - x) / scale, _
        '    (e.MarginBounds.Top - y) / scale, _
        '    e.MarginBounds.Width / scale, _
        '    e.MarginBounds.Height / scale)

        ' Draw controls in back to front order.
        For i As Integer = frm.Controls.Count - 1 To 0 Step -1
            DrawControl(frm.Controls(i), e.Graphics, 0, 0)
        Next i

        ' Draw a box around the form if desired.
        If draw_box Then
            e.Graphics.DrawRectangle(Pens.Black, 0, 0, frm.ClientSize.Width, frm.ClientSize.Height)
        End If
    End Sub

    ' Draw this control.
    Private Sub DrawControl(ByVal ctl As Control, ByVal gr As Graphics, ByVal x_off As Integer, ByVal y_off As Integer)
        ' Don't bother if the control is not visible.
        If Not ctl.Visible Then Exit Sub

        Dim l As Integer = x_off + ctl.Left
        Dim t As Integer = y_off + ctl.Top
        Dim w As Integer = ctl.Width
        Dim h As Integer = ctl.Height

        Select Case TypeName(ctl)
            Case "Button"
                DrawTextualControl(ctl, gr, l, t, w, h, CType(ctl, Button).TextAlign, True)
            Case "Label"
                DrawTextualControl(ctl, gr, l, t, w, h, CType(ctl, Label).TextAlign)
            Case "LinkLabel"
                DrawTextualControl(ctl, gr, l, t, w, h, CType(ctl, LinkLabel).TextAlign)
            Case "TextBox"
                DrawTextBox(ctl, gr, l, t, w, h)
            Case "PictureBox"
                DrawPictureBox(ctl, gr, l, t, w, h)
            Case "CheckBox"
                DrawCheckBox(ctl, gr, l, t, w, h)
            Case "RadioButton"
                DrawRadioButton(ctl, gr, l, t, w, h)
            Case "GroupBox"
                DrawGroupBox(ctl, gr, l, t, w, h)
            Case "ListBox"
                DrawListBox(ctl, gr, l, t, w, h)
            Case "ComboBox"
                DrawComboBox(ctl, gr, l, t, w, h)
            Case Else
                DrawUnknown(ctl, gr, l, t, w, h)
        End Select

        ' If this control contains others, draw them.
        If ctl.Controls.Count > 0 Then
            x_off += ctl.Left
            y_off += ctl.Top

            For i As Integer = ctl.Controls.Count - 1 To 0 Step -1
                DrawControl(ctl.Controls(i), gr, x_off, y_off)
            Next i
        End If
    End Sub

    ' Draw this control's border if it has one.
    Private Sub DrawBorder(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer, Optional ByVal force_border As Boolean = False)
        ' Blank the control's area.
        gr.FillRectangle(Brushes.White, l, t, w, h)

        If Not force_border Then
            ' See if this type of control 
            ' has a BorderStyle property.
            Dim ctl_type As Type = ctl.GetType
            Dim borderstyle_info As System.Reflection.PropertyInfo = ctl_type.GetProperty("BorderStyle")
            If Not (borderstyle_info Is Nothing) Then
                ' See if BorderStyle is None.
                force_border = (CType(borderstyle_info.GetValue(ctl, Nothing), BorderStyle) <> BorderStyle.None)
            End If
        End If

        ' Draw the border.
        If force_border Then gr.DrawRectangle(Pens.Black, l, t, w, h)
    End Sub

    ' Center some text in this control's space.
    Private Sub CenterText(ByVal txt As String, ByVal fnt As Font, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer, Optional ByVal draw_border As Boolean = False)
        DrawText(txt, fnt, ContentAlignment.MiddleCenter, gr, l, t, w, h)
    End Sub

    ' Draw some text in this control's space 
    ' appropriately aligned.
    Private Sub DrawText(ByVal txt As String, ByVal fnt As Font, ByVal align As ContentAlignment, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer, Optional ByVal draw_border As Boolean = False)
        Dim layout_rectangle As New RectangleF(l, t, w, h)
        Dim string_format As New StringFormat

        Select Case align
            Case ContentAlignment.BottomCenter, ContentAlignment.BottomLeft, ContentAlignment.BottomRight
                string_format.LineAlignment = StringAlignment.Far
            Case ContentAlignment.MiddleCenter, ContentAlignment.MiddleLeft, ContentAlignment.MiddleRight
                string_format.LineAlignment = StringAlignment.Center
            Case ContentAlignment.TopCenter, ContentAlignment.TopLeft, ContentAlignment.TopRight
                string_format.LineAlignment = StringAlignment.Near
        End Select

        Select Case align
            Case ContentAlignment.BottomCenter, ContentAlignment.MiddleCenter, ContentAlignment.TopCenter
                string_format.Alignment = StringAlignment.Center
            Case ContentAlignment.BottomLeft, ContentAlignment.MiddleLeft, ContentAlignment.TopLeft
                string_format.Alignment = StringAlignment.Near
            Case ContentAlignment.BottomRight, ContentAlignment.MiddleRight, ContentAlignment.TopRight
                string_format.Alignment = StringAlignment.Far
        End Select

        gr.DrawString(txt, fnt, Brushes.Black, _
            layout_rectangle, string_format)
    End Sub

    ' Draw a box for an unknown control.
    Private Sub DrawUnknown(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer)
        DrawBorder(ctl, gr, l, t, w, h, True)
        CenterText("???", ctl.Font, gr, l, t, w, h)
    End Sub

    ' Draw a control that contains text and may
    ' have a border.
    Private Sub DrawTextualControl(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer, ByVal align As ContentAlignment, Optional ByVal force_border As Boolean = False)
        DrawBorder(ctl, gr, l, t, w, h, force_border)
        DrawText(ctl.Text, ctl.Font, align, gr, l, t, w, h)
    End Sub

    ' Draw a TextBox.
    Private Sub DrawTextBox(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer)
        Dim txt As TextBox = CType(ctl, TextBox)
        Select Case txt.TextAlign
            Case HorizontalAlignment.Center
                DrawTextualControl(ctl, gr, l, t, w, h, ContentAlignment.TopCenter)
            Case HorizontalAlignment.Left
                DrawTextualControl(ctl, gr, l, t, w, h, ContentAlignment.TopLeft)
            Case HorizontalAlignment.Right
                DrawTextualControl(ctl, gr, l, t, w, h, ContentAlignment.TopRight)
        End Select
    End Sub

    ' Draw a PictureBox control.
    Private Sub DrawPictureBox(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer)
        DrawBorder(ctl, gr, l, t, w, h)

        ' Draw the picture if there is one.
        Dim pic As PictureBox = CType(ctl, PictureBox)
        If Not (pic.Image Is Nothing) Then
            Dim dest_rect As RectangleF
            Dim source_rect As RectangleF
            Select Case pic.SizeMode
                Case PictureBoxSizeMode.AutoSize, PictureBoxSizeMode.StretchImage
                    dest_rect = New RectangleF(l, t, w, h)
                    source_rect = New RectangleF(0, 0, pic.Image.Width, pic.Image.Height)
                Case PictureBoxSizeMode.CenterImage
                    dest_rect = New RectangleF
                    source_rect = New RectangleF
                    Dim x As Integer = (w - pic.Image.Width) \ 2
                    Dim y As Integer = (h - pic.Image.Height) \ 2
                    If x < 0 Then
                        dest_rect.X = l + 0
                        dest_rect.Width = w
                        source_rect.X = -x
                        source_rect.Width = w
                    Else
                        dest_rect.X = l + x
                        dest_rect.Width = pic.Image.Width
                        source_rect.X = 0
                        source_rect.Width = pic.Image.Width
                    End If
                    If y < 0 Then
                        dest_rect.Y = t + 0
                        dest_rect.Height = h
                        source_rect.Y = -y
                        source_rect.Height = h
                    Else
                        dest_rect.Y = t + y
                        dest_rect.Height = pic.Image.Height
                        source_rect.Y = 0
                        source_rect.Height = pic.Image.Height
                    End If
                Case PictureBoxSizeMode.Normal
                    dest_rect = New RectangleF(l, t, w, h)
                    source_rect = New RectangleF(0, 0, w, h)
            End Select

            gr.DrawImage(pic.Image, dest_rect, source_rect, GraphicsUnit.Pixel)
        End If
    End Sub

    ' Draw a CheckBox control. This is pretty simple.
    ' It doesn't worry about the stranger CheckAlign
    ' values such as MiddleCenter, doesn't handle 
    ' Appearance = Button, etc.
    Private Sub DrawCheckBox(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer)
        ' Erase behind the control.
        DrawBorder(ctl, gr, l, t, w, h)

        Const BOX_WID As Integer = 10
        Dim box_x As Integer
        Dim text_x As Integer
        Dim text_align As ContentAlignment

        ' See if the box belongs on the left or right.
        Dim check_box As CheckBox = CType(ctl, CheckBox)
        Select Case check_box.CheckAlign
            Case ContentAlignment.BottomRight, ContentAlignment.MiddleRight, ContentAlignment.TopRight
                box_x = l + w - BOX_WID
                text_x = l
                text_align = ContentAlignment.MiddleRight
            Case Else
                box_x = l
                text_x = CInt(l + BOX_WID * 1.5)
                text_align = ContentAlignment.MiddleLeft
        End Select

        ' Draw the text.
        DrawText(ctl.Text, ctl.Font, text_align, gr, _
            text_x, t, CInt(w - 1.5 * BOX_WID), h)

        ' Draw the box.
        Dim box_y As Integer = t + (h - BOX_WID) \ 2
        gr.DrawRectangle(Pens.Black, box_x, box_y, BOX_WID, BOX_WID)

        ' Draw an X if necessary.
        If check_box.Checked Then
            gr.DrawLine(Pens.Black, box_x, box_y, box_x + BOX_WID, box_y + BOX_WID)
            gr.DrawLine(Pens.Black, box_x, box_y + BOX_WID, box_x + BOX_WID, box_y)
        End If
    End Sub

    ' Draw a RadioButton control. This is pretty simple.
    ' It doesn't worry about the stranger CheckAlign
    ' values such as MiddleCenter, doesn't handle 
    ' Appearance = Button, etc.
    Private Sub DrawRadioButton(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer)
        ' Erase behind the control.
        DrawBorder(ctl, gr, l, t, w, h)

        Const CIRCLE_WID As Integer = 10
        Dim circle_x As Integer
        Dim text_x As Integer
        Dim text_align As ContentAlignment

        ' See if the circle belongs on the left or right.
        Dim radio_button As RadioButton = CType(ctl, RadioButton)
        Select Case radio_button.CheckAlign
            Case ContentAlignment.BottomRight, ContentAlignment.MiddleRight, ContentAlignment.TopRight
                circle_x = l + w - CIRCLE_WID
                text_x = l
                text_align = ContentAlignment.MiddleRight
            Case Else
                circle_x = l
                text_x = CInt(l + CIRCLE_WID * 1.5)
                text_align = ContentAlignment.MiddleLeft
        End Select

        ' Draw the text.
        DrawText(ctl.Text, ctl.Font, text_align, gr, _
            text_x, t, CInt(w - 1.5 * CIRCLE_WID), h)

        ' Draw the circle.
        Dim circle_y As Integer = t + (h - CIRCLE_WID) \ 2
        gr.DrawEllipse(Pens.Black, circle_x, circle_y, CIRCLE_WID, CIRCLE_WID)

        ' Fill the circle if necessary.
        If radio_button.Checked Then
            gr.FillEllipse(Brushes.Black, _
                circle_x + 2, circle_y + 2, _
                CIRCLE_WID - 4, CIRCLE_WID - 4)
        End If
    End Sub

    ' Draw a GroupBox control.
    Private Sub DrawGroupBox(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer)
        ' Get the text's size.
        Const INDENT As Integer = 5
        Dim text_size As SizeF = gr.MeasureString( _
            ctl.Text, ctl.Font, w - 2 * INDENT)

        ' Draw the border.
        DrawBorder(ctl, gr, l, _
            t + CInt(text_size.Height / 2), _
            w, h - CInt(text_size.Height / 2), True)

        ' Erase a spot for the text.
        gr.FillRectangle(Brushes.White, l + INDENT, t, _
            CInt(text_size.Width), CInt(text_size.Height))

        ' Draw the text.
        DrawText(ctl.Text, ctl.Font, _
            ContentAlignment.MiddleLeft, gr, _
            l + INDENT, t, CInt(text_size.Width), CInt(text_size.Height))
    End Sub

    ' Draw a ListBox control.
    Private Sub DrawListBox(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer)
        ' Draw the border.
        DrawBorder(ctl, gr, l, t, w, h)

        ' Display the items starting with
        ' the first visible item.
        Dim list_box As ListBox = CType(ctl, ListBox)
        Dim line_size As SizeF = gr.MeasureString("X", ctl.Font)
        Dim y As Integer = t + 2
        Dim layout_rectangle As New RectangleF(l + 2, y, w - 4, line_size.Height)
        Dim txt As String
        For i As Integer = list_box.TopIndex To list_box.Items.Count - 1
            ' Make sure we have room.
            If layout_rectangle.Y + layout_rectangle.Height > t + h Then Exit For

            ' Display this text.
            txt = CStr(list_box.Items(i))
            If list_box.SelectedItems.Contains(txt) Then
                gr.FillRectangle(Brushes.Black, _
                    CInt(layout_rectangle.Left), _
                    CInt(layout_rectangle.Top), _
                    CInt(layout_rectangle.Width), _
                    CInt(layout_rectangle.Height))
                gr.DrawString(txt, list_box.Font, _
                    Brushes.White, layout_rectangle)
            Else
                gr.DrawString(txt, list_box.Font, _
                    Brushes.Black, layout_rectangle)
            End If

            layout_rectangle.Y += layout_rectangle.Height
        Next i
    End Sub

    ' Draw a ComboBox control.
    Private Sub DrawComboBox(ByVal ctl As Control, ByVal gr As Graphics, ByVal l As Integer, ByVal t As Integer, ByVal w As Integer, ByVal h As Integer)
        DrawTextualControl(ctl, gr, l, t, w, h, ContentAlignment.TopLeft, True)
    End Sub
End Module
