Imports System.Collections.Generic
Imports System.Drawing.Imaging
Imports System.Drawing.Printing
Imports System.Math

Public Class Form1
    ' Display a print preview.
    Private Sub btnPreview_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPreview.Click
        ' Set the print preview control's PrintDocument object.
        pviewMetafile.Document = pdocMetafile

        ' Display the preview.
        ' The PrintDocument object does the real work.
        pviewMetafile.ShowDialog()
    End Sub

    ' Print.
    Private Sub btnPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrint.Click
        ' Use the PrintDocument to print.
        pdocMetafile.Print()
    End Sub

#Region "Printing Code"
    ' Metafiles representing the pages we will print.
    Private m_Pages As List(Of Metafile)

    ' The number of the next page we will print.
    Private m_NextPage As Integer

    ' Make the metafiles.
    Private Sub MakeMetafiles(ByVal page_settings As PageSettings)
        ' Get a Graphics object representing the printer settings.
        Dim printer_settings As PrinterSettings = pdocMetafile.PrinterSettings
        Dim measure_gr As Graphics = printer_settings.CreateMeasurementGraphics()
        Dim hdc As IntPtr = measure_gr.GetHdc()

        ' Get the page's bounding rectangle in inches.
        ' Note that the PageSettings are in hundredths of inches.
        Dim page_rect As New RectangleF( _
            CSng(page_settings.Bounds.Left / 100), _
            CSng(page_settings.Bounds.Top / 100), _
            CSng(page_settings.Bounds.Width / 100), _
            CSng(page_settings.Bounds.Height / 100))

        ' Get a rectangle bounding the drawing area
        ' inside the margins in hundredths of inches.
        Dim xmin As Single = page_settings.Bounds.Left + page_settings.Margins.Left
        Dim ymin As Single = page_settings.Bounds.Top + page_settings.Margins.Top
        Dim xmax As Single = page_settings.Bounds.Right - page_settings.Margins.Right
        Dim ymax As Single = page_settings.Bounds.Bottom - page_settings.Margins.Bottom
        Dim wid As Single = xmax - xmin
        Dim hgt As Single = ymax - ymin
        Dim margin_rect As New RectangleF(xmin, ymin, xmax - xmin, ymax - ymin)

        ' Make a list of Graphics objects for the metafiles.
        Dim grs As New List(Of Graphics)

        ' Make the pages.
        m_Pages = New List(Of Metafile)
        For i As Integer = 0 To 3
            ' Make a new metafile for this page.
            Dim mf As New Metafile(hdc, _
                page_rect, _
                MetafileFrameUnit.Inch, _
                EmfType.EmfOnly)
            m_Pages.Add(mf)

            ' Make a Graphics object for the metafile.
            Dim gr As Graphics = Graphics.FromImage(mf)
            grs.Add(gr)

            ' Draw the page.
            Select Case i
                Case 0  ' Red ellipse.
                    Using the_pen As New Pen(Color.Red, 5)
                        gr.DrawEllipse(the_pen, margin_rect)
                    End Using
                Case 1  ' Green diamond.
                    Dim cx As Single = (xmin + xmax) / 2
                    Dim cy As Single = (ymin + ymax) / 2
                    Dim pts() As PointF = { _
                        New PointF(cx, ymin), _
                        New PointF(xmax, cy), _
                        New PointF(cx, ymax), _
                        New PointF(xmin, cy) _
                    }
                    Using the_pen As New Pen(Color.Green, 10)
                        gr.DrawPolygon(the_pen, pts)
                    End Using
                Case 2  ' Blue hexagon.
                    Dim cx As Single = (xmin + xmax) / 2
                    Dim cy As Single = (ymin + ymax) / 2
                    Dim radius As Single = (xmax - xmin) / 2
                    Dim theta As Single = -PI / 2
                    Dim dtheta As Single = 2 * PI / 6
                    Dim pts(0 To 5) As PointF
                    For pt_num As Integer = 0 To 5
                        pts(pt_num) = New PointF( _
                            CSng(cx + radius * Cos(theta)), _
                            CSng(cy + radius * Sin(theta)))
                        theta += dtheta
                    Next pt_num
                    Using the_pen As New Pen(Color.Blue, 20)
                        gr.DrawPolygon(the_pen, pts)
                    End Using
                Case 3  ' Orange star.
                    Dim cx As Single = (xmin + xmax) / 2
                    Dim cy As Single = (ymin + ymax) / 2
                    Dim radius As Single = (xmax - xmin) / 2
                    Dim theta As Single = -PI / 2
                    Dim dtheta As Single = 2 * 2 * PI / 5
                    Dim pts(0 To 4) As PointF
                    For pt_num As Integer = 0 To 4
                        pts(pt_num) = New PointF( _
                            CSng(cx + radius * Cos(theta)), _
                            CSng(cy + radius * Sin(theta)))
                        theta += dtheta
                    Next pt_num
                    Using the_pen As New Pen(Color.Orange, 30)
                        gr.DrawPolygon(the_pen, pts)
                    End Using
            End Select
        Next i

        ' Now that we know the page count, draw the page numbers.
        Dim string_format As New StringFormat
        string_format.Alignment = StringAlignment.Center
        string_format.LineAlignment = StringAlignment.Center
        Using big_font As New Font(Me.Font.FontFamily, 70, FontStyle.Bold, GraphicsUnit.Pixel)
            Dim num_pages As Integer = m_Pages.Count
            Dim margin_rectf As New RectangleF(xmin, ymin, xmax - xmin, ymax - ymin)
            For i As Integer = 0 To m_Pages.Count - 1
                grs(i).DrawString("Page " & i + 1 & " of " & num_pages, _
                    big_font, Brushes.Black, _
                    margin_rectf, string_format)
                grs(i).Dispose()

                ' Save the metafile.
                ' m_Pages(i).Save(Application.StartupPath & "/page" & i & ".wmf", ImageFormat.Wmf)
            Next i
        End Using ' big_font

        ' Clean up.
        measure_gr.ReleaseHdc(hdc)
        measure_gr.Dispose()
    End Sub

    ' Prepare to print.
    Private Sub pdlgWrap_BeginPrint(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintEventArgs) Handles pdocMetafile.BeginPrint
        ' Make the metafiles.
        MakeMetafiles(pdocMetafile.PrinterSettings.DefaultPageSettings)

        ' Start with the first page.
        m_NextPage = 0
    End Sub


    ' Change to True to draw margins, picture areas, and layout areas.
#Const DEBUGGING = True

    ' The PrintDocument needs a page. Print the next one.
    Private Sub pdlgWrap_PrintPage(ByVal sender As System.Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles pdocMetafile.PrintPage
        ' Draw this page's metafile.
        e.Graphics.DrawImage(m_Pages(m_NextPage), 0, 0, e.PageBounds.Width, e.PageBounds.Height)

#If DEBUGGING Then
        ' Draw the margins.
        e.Graphics.DrawRectangle(Pens.Red, e.MarginBounds)
#End If

        ' Move to the next page.
        m_NextPage += 1
        e.HasMorePages = (m_NextPage <= m_Pages.Count - 1)
    End Sub
#End Region ' Printing Code

End Class
